"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const parser = require("./parser");
const ns_capabilities_1 = require("./interfaces/ns-capabilities");
const capabilities_helper_1 = require("./capabilities-helper");
const utils_1 = require("./utils");
class NsCapabilities {
    constructor() {
        this.exceptions = new Array();
        this._projectDir = parser.projectDir;
        this._projectBinary = parser.projectBinary;
        this._pluginRoot = parser.pluginRoot;
        this._pluginBinary = parser.pluginBinary;
        this._appPath = parser.appPath;
        this._port = parser.port;
        this._verbose = parser.verbose;
        this._appiumCapsLocation = parser.appiumCapsLocation;
        this._appiumCaps = capabilities_helper_1.resolveCapabilities(this._appiumCapsLocation, parser.runType, parser.projectDir);
        this._testFolder = parser.testFolder;
        this._storage = parser.storage;
        this._testReports = parser.testReports;
        this._reuseDevice = parser.reuseDevice;
        this._devMode = parser.devMode;
        this._runType = parser.runType;
        this._isAndroid = this.isAndroidPlatform();
        this._isIOS = !this._isAndroid;
        this._isSauceLab = parser.isSauceLab;
        this._ignoreDeviceController = parser.ignoreDeviceController;
        this._wdaLocalPort = parser.wdaLocalPort;
        this._path = parser.path;
        this._relaxedSecurity = parser.relaxedSecurity;
        this._cleanApp = parser.cleanApp;
        this._attachToDebug = parser.attachToDebug;
        this._sessionId = parser.sessionId;
        this._startSession = parser.startSession;
        this.setAutomationName();
        this.resolveApplication();
        this.checkMandatoryCapabiliies();
        this.throwExceptions();
        this.shouldSetFullResetOption();
    }
    get path() { return this._path; }
    get projectDir() { return this._projectDir; }
    get projectBinary() { return this._projectBinary; }
    get pluginRoot() { return this._pluginRoot; }
    get pluginBinary() { return this._pluginBinary; }
    get port() { return this._port; }
    get verbose() { return this._verbose; }
    get appiumCapsLocation() { return this._appiumCapsLocation; }
    get appiumCaps() { return this._appiumCaps; }
    get testFolder() { return this._testFolder; }
    get storage() { return this._storage; }
    get testReports() { return this._testReports; }
    get reuseDevice() { return this._reuseDevice; }
    get devMode() { return this._devMode; }
    get runType() { return this._runType; }
    get isAndroid() { return this._isAndroid; }
    get isIOS() { return this._isIOS; }
    get isSauceLab() { return this._isSauceLab; }
    get automationName() { return this._automationName; }
    get appPath() { return this._appPath; }
    get appName() { return this._appName; }
    set appName(appName) { this._appName = appName; }
    get ignoreDeviceController() { return this._ignoreDeviceController; }
    set ignoreDeviceController(ignoreDeviceController) { this._ignoreDeviceController = ignoreDeviceController; }
    get wdaLocalPort() { return this._wdaLocalPort; }
    get device() { return this._device; }
    set device(device) { this._device = device; }
    get emulatorOptions() { return (this._emulatorOptions || "-wipe-data -gpu on"); }
    get relaxedSecurity() { return this._relaxedSecurity; }
    get cleanApp() { return this._cleanApp; }
    get attachToDebug() { return this._attachToDebug; }
    get sessionId() { return this._sessionId; }
    set sessionId(sessionId) { this._sessionId = sessionId; }
    get startSession() { return this._startSession; }
    isAndroidPlatform() { return this._appiumCaps.platformName.toLowerCase().includes("android"); }
    shouldSetFullResetOption() {
        if (this._ignoreDeviceController) {
            this.appiumCaps["fullReset"] = true;
            this.appiumCaps["noReset"] = false;
            console.log("Changing appium setting fullReset: true and noReset: false ");
        }
        if (this._attachToDebug || this._devMode) {
            this.appiumCaps["fullReset"] = false;
            this.appiumCaps["noReset"] = true;
            console.log("Changing appium setting fullReset: false and noReset: true ");
        }
    }
    setAutomationName() {
        if (this.appiumCaps["automationName"]) {
            switch (this.appiumCaps["automationName"].toLowerCase()) {
                case ns_capabilities_1.AutomationName.UiAutomator2.toString().toLowerCase():
                    this._automationName = ns_capabilities_1.AutomationName.UiAutomator2;
                    break;
                case ns_capabilities_1.AutomationName.Appium.toString().toLowerCase():
                    this._automationName = ns_capabilities_1.AutomationName.Appium;
                    break;
                case ns_capabilities_1.AutomationName.XCUITest.toString().toLowerCase():
                    this._automationName = ns_capabilities_1.AutomationName.XCUITest;
                    break;
            }
        }
        else {
            if (this._isAndroid) {
                if (this.tryGetAndroidApiLevel() > 6 || (this.appiumCaps["apiLevel"] && this.appiumCaps["apiLevel"].toLowerCase().includes("p"))) {
                    this._automationName = ns_capabilities_1.AutomationName.UiAutomator2;
                }
            }
        }
        if (this._automationName) {
            this.appiumCaps["automationName"] = this._automationName.toString();
            utils_1.logInfo(`Automation name set to: ${this.appiumCaps["automationName"]}`);
            console.log(`To change automation name, you need to set it in appium capabilities!`);
        }
        else {
            console.log(`Appium will use default automation name`);
        }
    }
    tryGetAndroidApiLevel() {
        try {
            if (this.appiumCaps["platformVersion"]) {
                const apiLevel = this.appiumCaps["platformVersion"].split(".").splice(0, 2).join('.');
                return parseFloat(apiLevel);
            }
        }
        catch (error) { }
        return undefined;
    }
    resolveApplication() {
        if (this.isSauceLab) {
            this._appiumCaps.app = `sauce-storage:${this.appPath}`;
            this._ignoreDeviceController = true;
            console.log("Using Sauce Labs. The application path is changed to: " + this.appPath);
        }
        else {
            this.appiumCaps.app = utils_1.getAppPath(this);
            this._appPath = this._appiumCaps.app;
            console.log("Application full path: " + this._appiumCaps.app);
        }
    }
    checkMandatoryCapabiliies() {
        if (!this.isSauceLab && !utils_1.fileExists(this._appiumCaps.app)) {
            this.exceptions.push("The application folder doesn't exist!");
        }
        if (!this._runType) {
            this.exceptions.push("Missing runType! Please select one from appium capabilities file!");
        }
        if (!this._appiumCaps.platformName) {
            this.exceptions.push("Platform name is missing! Please, check appium capabilities file!");
        }
        if (!this._appiumCaps.platformVersion) {
            console.warn("Platform version is missing! You'd better to set it in order to use the correct device");
        }
        if (!this._appiumCaps.deviceName && !this._appiumCaps.udid) {
            this.exceptions.push("The device name or udid are missing! Please, check appium capabilities file!");
        }
    }
    throwExceptions() {
        this.exceptions.forEach(msg => {
            utils_1.logError(msg);
        });
        if (this.exceptions.length > 0) {
            process.exit(1);
        }
    }
}
exports.NsCapabilities = NsCapabilities;
//# sourceMappingURL=ns-capabilities.js.map